<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 01/11/19
 * Time: 18.05
 */

namespace FiloBlu\Rma\Plugin\Model;
use Exception;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use Psr\Log\LoggerInterface;
use FiloBlu\Rma\Helper\ChangeSizeOrder;

class ChangeSizeOrderPlugin
{
        
 
    protected $changSizeItems = [];

    /**
     * @var LoggerInterface
     */
    protected $logger;
    
    /**
     * @var ChangeSizeOrder
     */
    protected $changeSizeOrder;

    /**
     * @var ChangeSizeAttributeHelper
     */
    protected $changeSizeAttributeHelper;


    /**
     * ChangeSizeOrderPlugin constructor.
     * @param LoggerInterface $logger
     * @param ChangeSizeOrder $changeSizeOrder
     * @param ChangeSizeAttributeHelper $changeSizeAttributeHelper
     */
    public function __construct(LoggerInterface $logger,
                                ChangeSizeOrder $changeSizeOrder,
                                ChangeSizeAttributeHelper $changeSizeAttributeHelper)
    {
        $this->logger = $logger;
        $this->changeSizeOrder = $changeSizeOrder;
        $this->changeSizeAttributeHelper =$changeSizeAttributeHelper;
    }


    /**
     * @param \Magento\Rma\Model\Rma $subject
     * @param $result
     * @return mixed
     */
    public function afterSaveRma(\Magento\Rma\Model\Rma $subject, $result)
    {

        try {

            if ($result && isset($result['items'])) {

                $rmaItems = $result['items'];
                $parentProduct = null;

                foreach ($rmaItems as $item) {
                    if ($item->getStatus() == \Magento\Rma\Model\Rma\Source\Status::STATE_PENDING && $item->getResolution() == $this->changeSizeAttributeHelper->getChangeSizeAttributeOptionId()) {
                        $brandReasonId = isset($item['reason']) ? $item['reason'] : 0;
                        $standardReasonId = isset($item['reason']) ? $item['reason'] : 0;
                        $conditionId = isset($item['condition']) ? $item['condition'] : 0;

                        // check if customer
                        if (isset($item->getData()[ChangeSizeAttributeHelper::CHANGE_SIZE_RMA_ATTRIBUTE_CODE])
                            && $item->getData()[ChangeSizeAttributeHelper::CHANGE_SIZE_RMA_ATTRIBUTE_CODE] != "") {
                            $changeProductId = $item->getData()[ChangeSizeAttributeHelper::CHANGE_SIZE_RMA_ATTRIBUTE_CODE];
                            $parentProduct = $item->getProductSku();
                            $this->changSizeItems = array(
                                'product_id' => $changeProductId,
                                'qty' => 1,
                                'parent_product_sku' => $item->getProductSku(),
                                'rma_entity_id' => $item->getEntityId(),
                                'original_order_item_id' => $item->getOrderItemId()
                            );
                        }
                        $customEmail = $subject->getCustomerCustomEmail();
                        if (isset($customEmail) && $customEmail != "") {
                            $subject->getOrder()->setCustomerEmail($customEmail);
                        }
                        // create an order for each item requested
                        $qty = $item['qty_requested'];
                        for ($c = 0; $c < $qty; $c++) {
                            $this->createChangeSizeOrder($subject->getOrder(), $this->changSizeItems, $subject->getEntityId(), $parentProduct);
                        }
                    }

                }
            }
        }
        catch (\NoSuchElementException $e){
            $this->logger->critical($e->getMessage(),['exception']);
        }
        catch (\Exception $e){
            $this->logger->critical($e->getMessage(),['exception']);
        }
        return $result;
    }

    /**
     * 
     * @param \Magento\Sales\Model\Order $order
     * @param array $items
     * @param int $rmaId
     * @param string|null $parentProduct
     */
    protected function createChangeSizeOrder($order, array $items, $rmaId, $parentProduct = null){

        $orderData =[
            'order' => $order,
            'items' => $items,
            'rma_id'=> $rmaId,
            'parent_product_sku' => $parentProduct
        ];
        
        if($parentProduct !== null && is_numeric($items['product_id'])):
            $this->changeSizeOrder->createMageOrder($orderData);
        endif;
        
    }
}